"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMetaplexTokenMetadata = exports.toMetadata = void 0;
const react_async_hook_1 = require("react-async-hook");
function toMetadata(tokenInfo) {
    if (!tokenInfo) {
        return undefined;
    }
    return ({
        updateAuthority: "",
        mint: tokenInfo.address,
        data: ({
            name: tokenInfo.name,
            symbol: tokenInfo.symbol,
            uri: tokenInfo.logoURI || "",
            creators: null,
            sellerFeeBasisPoints: 0,
            // @ts-ignore
            collection: null,
            // @ts-ignore
            uses: null,
        }),
        primarySaleHappened: false,
        isMutable: false,
        editionNonce: null,
    });
}
exports.toMetadata = toMetadata;
/**
 * Get the token account and all metaplex metadata around the token
 *
 * @param token
 * @returns
 */
function useMetaplexTokenMetadata(token) {
    const { result: metadataAccountKey, loading, error, } = (0, react_async_hook_1.useAsync)((token) => __awaiter(this, void 0, void 0, function* () {
        if (!token) {
            return null;
        }
        const url = `https://mainnet.helius-rpc.com/?api-key=0d4b4fd6-c2fc-4f55-b615-a23bab1ffc85`;
        try {
            const response = yield fetch(url, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    jsonrpc: '2.0',
                    id: 'my-id',
                    method: 'getAsset',
                    params: {
                        id: token.toString(),
                    },
                }),
            });
            const { result } = yield response.json();
            console.log("Asset Data: ", result);
            if (result) {
                return {
                    mint: result.mint,
                    name: result.content.metadata.name,
                    symbol: result.content.metadata.symbol,
                    description: result.content.metadata.description,
                    image: result.content.links.image,
                    decimals: result.token_info.decimals,
                    supply: result.token_info.supply,
                };
            }
            else {
                console.log('No asset metadata found');
                return null;
            }
        }
        catch (error) {
            console.error('Error fetching asset:', error);
            throw error;
        }
    }), [token]);
    // @ts-ignore
    return {
        // @ts-ignore
        loading: Boolean(
        // @ts-ignore
        token && (loading)),
        displayName: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.name,
        error: error,
        mint: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.mint,
        // @ts-ignore
        metadata: metadataAccountKey,
        // @ts-ignore
        metadataKey: metadataAccountKey,
        data: {
            name: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.name,
            symbol: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.symbol,
            description: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.description,
            image: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.image,
            // @ts-ignore
            creators: null,
            // @ts-ignore
            external_url: null,
            // @ts-ignore
            seller_fee_basis_points: 0,
            // @ts-ignore
            properties: null,
            // @ts-ignore
            collection: null,
            // @ts-ignore
            uses: null,
        },
        image: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.image,
        description: metadataAccountKey === null || metadataAccountKey === void 0 ? void 0 : metadataAccountKey.description,
        // @ts-ignore
    };
}
exports.useMetaplexTokenMetadata = useMetaplexTokenMetadata;
//# sourceMappingURL=useMetaplexMetadata.js.map