"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTokenMetadata = void 0;
const react_async_hook_1 = require("react-async-hook");
/**
 * Get the token account and all metaplex + token collective metadata around the token
 *
 * @param token
 * @returns
 */
function useTokenMetadata(token) {
    const fetchMetadata = () => __awaiter(this, void 0, void 0, function* () {
        if (!token)
            return null;
        const url = `https://mainnet.helius-rpc.com/?api-key=0d4b4fd6-c2fc-4f55-b615-a23bab1ffc85`;
        try {
            const response = yield fetch(url, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    jsonrpc: '2.0',
                    id: 'my-id',
                    method: 'getAsset',
                    params: {
                        id: token.toString(),
                    },
                }),
            });
            const { result } = yield response.json();
            if (result) {
                return {
                    mint: result.mint,
                    name: result.content.metadata.name,
                    symbol: result.content.metadata.symbol,
                    description: result.content.metadata.description,
                    image: result.content.links.image,
                    decimals: result.token_info.decimals,
                    supply: result.token_info.supply,
                };
            }
        }
        catch (error) {
            console.error('Error fetching asset:', error);
            throw error;
        }
    });
    const { result: metadata, loading, error } = (0, react_async_hook_1.useAsync)(fetchMetadata, [token]);
    return Object.assign(Object.assign({}, metadata), { loading,
        error, account: undefined });
}
exports.useTokenMetadata = useTokenMetadata;
//# sourceMappingURL=useTokenMetadata.js.map